/*
 * Copyright (C) 2013-2016 CERN (www.cern.ch)
 * Author: Tomasz Wlostowski <tomasz.wlostowski@cern.ch>
 * Author: Federico Vaga <federico.vaga@cern.ch>
 *
 * SPDX-License-Identifier: LGPL-3.0-or-later
 */

#include <stdio.h>
#include <stdint.h>
#include <string.h>

#include "mockturtle-rt.h"


/**
 * It sends a character to the UART interface
 * @param[in] c the character to sent
 * @return 0 on success (for the time being it does not fail)
 */
int putchar(int c)
{
	lr_writel(c, MT_CPU_LR_REG_UART_CHR);

	return 0;
}


/**
 * It sends a string over the serial interface
 * @param[in] p string to send
 * @return number of sent characters
 *
 * The function does not add any special character.
 * If you neeed the new-line or the carriage-return
 * you have to add them to your strings.
 */
int puts(const char *p)
{
	const char *p_orig = p;
	char c;

	while ((c = *(p++)))
		putchar(c);

	return p - p_orig;
}



/**
 * It prints on the serial interface the given message header
 * @param[in] h message header to print
 */
void trtl_fw_print_header(struct trtl_hmq_header *h)
{
	pr_debug("\tapp_id 0x%04"PRIx16" | flags   0x%02"PRIx8" | msg_id 0x%02"PRIx8"\r\n",
		 h->rt_app_id, h->flags, h->msg_id);
	delay(1000);
	pr_debug("\tlen 0x%04"PRIx16" | sync_id 0x%04"PRIx16"\r\n",
		 h->len, h->sync_id);
	delay(1000);
	pr_debug("\tseq 0x%08"PRIx32"\r\n",
		 h->seq);
	delay(1000);
}

/**
 * It prints on the serial interface a buffer
 * @param[in] d buffer data
 * @param[in] count number of 32bit words to show
 */
void trtl_fw_print_data(uint32_t *d, unsigned int count)
{
	int i;

	for (i = 0; i < count; i++) {
		pr_debug("%s: data[%d] = 0x%"PRIx32"\n\r", __func__, i , d[i]);
		delay(1000);
	}
}


/**
 * It prints on the serial console the given message
 * @param[in] msg a mock turtle message
 */
void trtl_fw_print_message(struct trtl_fw_msg *msg)
{
	pr_debug("Message\n\r");
	trtl_fw_print_header(msg->header);
	trtl_fw_print_data(msg->payload, msg->header->len);
}
